==> spread-5.5.0.34-py3-none-any.whl: spread_core/bam/equipment.py <==
import json
import logging

from spread_core.bam.recipe import Recipe

import spread_core.errors.project_errors as errors
import spread_core.mqtt as mqtt
from spread_core.bam.const import *
from spread_core.tools.manager_interface import ManagerOfBroker
from spread_core.warehouse.equipment_warehouse import EquipmentWarehouse
from . import engineries, subgineries

REPLY_ON_ERROR = False


def extrude_ids(data: dict) -> [int, str]:
    return data.pop(ID, None), data.pop(TYPE, None)


def log_command(fn):
    def log(instance, topic, variable):
        logging.debug(
            f'COMMAND({variable.session_id if variable.session_id else ""})'
            f': {topic.__class__.__name__} {topic.entity_address.type}({topic.entity_address.id})'
            f'.{topic.entity_address.funit_type}' +
            (f'={variable.value}' if isinstance(topic, mqtt.spread.topic.Set) else '') +
            (f' params: {variable.params}' if variable.params else '')
        )
        fn(instance, topic, variable)

    return log


class Equipment(ManagerOfBroker, EquipmentWarehouse):
    subginery_type = None
    state_topics = None
    enginery_types = None

    def __init__(self, project_id, mqttc):
        super(Equipment, self).__init__(mqttc)
        EquipmentWarehouse.__init__(self, project_id)

        self.id = project_id
        self.loaded = False

    def start(self):
        if self.loaded:
            for entity in self.all_entities():
                values = {**entity.values}
                entity.values = {}
                for funit_type, value in values.items():
                    entity.publish_state(funit_type=funit_type, value=value)

            self.exploit_topics(self.subscribe)
        else:
            self.subscribe(mqtt.project.ProjectFile.managers(self.id))

    def on_project(self, topic, data):
        self.unsubscribe(topic)
        data = json.loads(data)

        if mqtt.project.ProjectFile.MANAGERS in data:
            self.on_project_managers(data[mqtt.project.ProjectFile.MANAGERS])

        elif mqtt.project.ProjectFile.PROVIDERS in data:
            self.on_project_providers(data[mqtt.project.ProjectFile.PROVIDERS])

        elif mqtt.project.ProjectFile.ENGINERIES in data:
            self.on_project_engineries(data[mqtt.project.ProjectFile.ENGINERIES])

        elif mqtt.project.ProjectFile.SUBGINERIES in data:
            self.on_project_subgineries(data[mqtt.project.ProjectFile.SUBGINERIES])
            self.on_ready()

    def on_project_managers(self, data):
        for item in data:
            self.on_project_manager(item)

        self.subscribe(mqtt.project.ProjectFile.providers(self.id))

    def on_project_manager(self, data: dict):
        try:
            self.add_manager(data[ID], data[TYPE])
        except Exception as ex:
            logging.warning(ex)

    def on_project_providers(self, data):
        for item in data:
            self.on_project_provider(item)

        self.subscribe(mqtt.project.ProjectFile.engineries(self.id))

    def on_project_provider(self, data):
        try:
            _id, _type = extrude_ids(data)

            manager = self.get_manager(data[MANAGER_ID])
            recipe = Recipe(self._project_id, manager[TYPE], manager[ID], _type, _id)

            self.add_provider(recipe)

        except Exception as ex:
            logging.warning(ex)

    def on_project_subgineries(self, data):
        for item in data:
            self.on_project_subginery(item)

    def on_project_subginery(self, data: dict):
        try:
            from spread_core.bam.generator import generate_subginery

            if data['type'] == self.subginery_type.__name__:
                _id, _type = extrude_ids(data)

                if _id is None or _type is None:
                    logging.warning(f'Empty subginery data of {_type}({_id})')
                else:
                    subginery = generate_subginery(_id, _type, data)
                    self.add_subginery(subginery)
        except Exception as ex:
            logging.warning(ex)

    def on_project_engineries(self, data):
        """ Apply engineries from project """

        for item in data:
            self.on_project_enginery(item)

        self.subscribe(mqtt.project.ProjectFile.subgineries(self.id))

    def on_project_enginery(self, data):
        allowed_types = [t.__name__ for t in self.enginery_types]

        try:
            entity_id, entity_type = extrude_ids(data)

            if entity_type not in allowed_types:
                return

            elif entity_id is None or entity_type is None:
                logging.warning(f'Empty enginery data of {entity_type}({entity_id})')

            else:
                from spread_core.bam.generator import generate_enginery

                if RECIPE in data and TYPE in data[RECIPE]:
                    eng = generate_enginery(entity_id, entity_type, data)

                    ingredients = sorted(data[RECIPE].get(INGREDIENTS, []), key=lambda ing: ing.get('index', -1))

                    for ingredient_data in ingredients:
                        try:
                            self.add_enginery(eng, ingredient_data[ID], ingredient_data.get(TYPE))
                        except ValueError as ex:
                            logging.warning(ex)

        except Exception as ex:
            logging.warning(ex)

    def on_ready(self):
        for entity in self.all_entities():
            entity.get_info()

        self.loaded = True
        logging.info('Project({}) loaded'.format(self.id))

        self.exploit_topics(self.subscribe)

    def exploit_topics(self, action):
        """Make action with topics"""

        for fut in self.state_topics or ():
            action(mqtt.spread.topic.State(mqtt.spread.address.ProviderAddress.broadcast(self.id, funit_type=fut)))

        for entity in (self.subginery_type, *(self.enginery_types or [])):
            for funit in entity.settable_units or []:
                action(mqtt.spread.topic.Set(entity.get_address_broadcast(self.id, funit)))

            for funit in entity.gettable_units or []:
                action(mqtt.spread.topic.Renew(entity.get_address_broadcast(self.id, funit)))

    @log_command
    def on_command(self, topic: mqtt.spread.topic.SpreadTopic, variable: mqtt.spread.variable.Variable):
        try:
            entity = self.get_entity(topic.entity_address.id)
            entity.on_command(topic, variable)

        except (errors.EntityNotPresent, errors.ClassifierError, errors.CommandError) as ex:
            logging.debug(ex)

            if REPLY_ON_ERROR and variable.session_id:
                to_topic = mqtt.spread.topic.Reply(self.id, variable.session_id)
                self.publish(to_topic, dict(error=dict(code=123, message='{}'.format(ex))), False)

    def on_state(self, topic: mqtt.spread.topic.State, variable: mqtt.spread.variable.Variable):
        recipe = self.get_recipe(topic.entity_address.manager_id, topic.entity_address.id)

        if recipe:
            recipe.dispatch_state(topic, variable)

    def on_exit(self):
        super(Equipment, self).on_exit()

        for entity in self.all_entities():
            try:
                entity.set_invalid()
            except BaseException as ex:
                logging.exception(ex)


class LightingEquipment(Equipment):
    subginery_type = subgineries.lighting.Lighting
    state_topics = ('On', 'BrightnessLevel', 'Temperature', 'EngineryHoldTime',
                    'CurrentLuminosity', 'CurrentPresence', 'LastEvent', 'Color')
    enginery_types = [
        engineries.DimmingLight, engineries.SwitchingLight, engineries.TunableWhiteLight,
        engineries.PresenceSensor, engineries.LightSensor,
        engineries.LightingArea, engineries.RgbLight, engineries.RgbwLight
    ]

    def __init__(self, *args, **kwargs):
        super(LightingEquipment, self).__init__(*args, **kwargs)
        self.button_presets = dict()

    def on_local(self, topic: mqtt.project.Preset, data):
        assert topic.entity_type == engineries.lighting_area.ButtonPreset.__name__

        if data:
            self.button_presets.setdefault(topic.entity_id, engineries.lighting_area.ButtonPreset(topic.entity_id))
            self.button_presets[topic.entity_id].update(json.loads(data))
        else:
            self.button_presets.pop(topic.entity_id)

    def exploit_topics(self, action):
        super(LightingEquipment, self).exploit_topics(action)
        action(mqtt.project.Preset.broadcast(self.id, engineries.lighting_area.ButtonPreset.__name__, '#'))


class ShadingEquipment(Equipment):
    subginery_type = subgineries.shading.Shading
    state_topics = ('PositionLevel',)
    enginery_types = [engineries.shade.Shutter, engineries.shade.Curtain, engineries.shade.Blind]


class AlarmEquipment(Equipment):
    subginery_type = subgineries.alarming.Alarm
    state_topics = ('Guard', 'Triggered')
    enginery_types = [
        engineries.alarm.FireSensor,
        engineries.alarm.IntruderSensor,
        engineries.alarm.LeakageSensor
    ]

==> spread-5.6.1.59-py3-none-any.whl: spread_core/bam/equipment.py <==
import json
import logging

from spread_core.bam.recipe import Recipe

import spread_core.errors.project_errors as errors
import spread_core.mqtt as mqtt
from spread_core.bam.const import *
from spread_core.tools.manager_interface import ManagerOfBroker
from spread_core.warehouse.equipment_warehouse import EquipmentWarehouse
from . import engineries, subgineries

REPLY_ON_ERROR = False


def extrude_ids(data: dict) -> [int, str]:
    return data.pop(ID, None), data.pop(TYPE, None)


def log_command(fn):
    def log(instance, topic, variable):
        logging.debug(
            f'COMMAND({variable.session_id if variable.session_id else ""})'
            f': {topic.__class__.__name__} {topic.entity_address.type}({topic.entity_address.id})'
            f'.{topic.entity_address.funit_type}' +
            (f'={variable.value}' if isinstance(topic, mqtt.spread.topic.Set) else '') +
            (f' params: {variable.params}' if variable.params else '')
        )
        fn(instance, topic, variable)

    return log


class Equipment(ManagerOfBroker, EquipmentWarehouse):
    subginery_type = None
    state_topics = None
    enginery_types = None

    def __init__(self, project_id, mqttc):
        super(Equipment, self).__init__(mqttc)
        EquipmentWarehouse.__init__(self, project_id)

        self.id = project_id
        self.loaded = False

    def start(self):
        if self.loaded:
            for entity in self.all_entities():
                values = {**entity.values}
                entity.values = {}
                for funit_type, value in values.items():
                    entity.publish_state(funit_type=funit_type, value=value)

            self.exploit_topics(self.subscribe)
        else:
            self.subscribe(mqtt.project.ProjectFile.managers(self.id))

    def on_project(self, topic, data):
        self.unsubscribe(topic)
        data = json.loads(data)

        if mqtt.project.ProjectFile.MANAGERS in data:
            self.on_project_managers(data[mqtt.project.ProjectFile.MANAGERS])

        elif mqtt.project.ProjectFile.PROVIDERS in data:
            self.on_project_providers(data[mqtt.project.ProjectFile.PROVIDERS])

        elif mqtt.project.ProjectFile.ENGINERIES in data:
            self.on_project_engineries(data[mqtt.project.ProjectFile.ENGINERIES])

        elif mqtt.project.ProjectFile.SUBGINERIES in data:
            self.on_project_subgineries(data[mqtt.project.ProjectFile.SUBGINERIES])
            self.on_ready()

    def on_project_managers(self, data):
        for item in data:
            self.on_project_manager(item)

        self.subscribe(mqtt.project.ProjectFile.providers(self.id))

    def on_project_manager(self, data: dict):
        try:
            self.add_manager(data[ID], data[TYPE])
        except Exception as ex:
            logging.warning(ex)

    def on_project_providers(self, data):
        for item in data:
            self.on_project_provider(item)

        self.subscribe(mqtt.project.ProjectFile.engineries(self.id))

    def on_project_provider(self, data):
        try:
            _id, _type = extrude_ids(data)

            manager = self.get_manager(data[MANAGER_ID])
            recipe = Recipe(self._project_id, manager[TYPE], manager[ID], _type, _id)

            self.add_provider(recipe)

        except Exception as ex:
            logging.warning(ex)

    def on_project_subgineries(self, data):
        for item in data:
            self.on_project_subginery(item)

    def on_project_subginery(self, data: dict):
        try:
            from spread_core.bam.generator import generate_subginery

            if data['type'] == self.subginery_type.__name__:
                _id, _type = extrude_ids(data)

                if _id is None or _type is None:
                    logging.warning(f'Empty subginery data of {_type}({_id})')
                else:
                    subginery = generate_subginery(_id, _type, data)
                    self.add_subginery(subginery)
        except Exception as ex:
            logging.warning(ex)

    def on_project_engineries(self, data):
        """ Apply engineries from project """

        for item in data:
            self.on_project_enginery(item)

        self.subscribe(mqtt.project.ProjectFile.subgineries(self.id))

    def on_project_enginery(self, data):
        allowed_types = [t.__name__ for t in self.enginery_types]

        try:
            entity_id, entity_type = extrude_ids(data)

            if entity_type not in allowed_types:
                return

            elif entity_id is None or entity_type is None:
                logging.warning(f'Empty enginery data of {entity_type}({entity_id})')

            else:
                from spread_core.bam.generator import generate_enginery

                if RECIPE in data and TYPE in data[RECIPE]:
                    eng = generate_enginery(entity_id, entity_type, data)

                    ingredients = sorted(data[RECIPE].get(INGREDIENTS, []), key=lambda ing: ing.get('index', -1))

                    for ingredient_data in ingredients:
                        try:
                            self.add_enginery(eng, ingredient_data[ID], ingredient_data.get(TYPE))
                        except ValueError as ex:
                            logging.warning(ex)

        except Exception as ex:
            logging.warning(ex)

    def on_ready(self):
        for entity in self.all_entities():
            entity.get_info()

        self.loaded = True
        logging.info('Project({}) loaded'.format(self.id))

        self.exploit_topics(self.subscribe)

    def exploit_topics(self, action):
        """Make action with topics"""

        for fut in self.state_topics or ():
            action(mqtt.spread.topic.State(mqtt.spread.address.ProviderAddress.broadcast(self.id, funit_type=fut)))

        for entity in (self.subginery_type, *(self.enginery_types or [])):
            for funit in entity.settable_units or []:
                action(mqtt.spread.topic.Set(entity.get_address_broadcast(self.id, funit)))

            for funit in entity.gettable_units or []:
                action(mqtt.spread.topic.Renew(entity.get_address_broadcast(self.id, funit)))

    @log_command
    def on_command(self, topic: mqtt.spread.topic.SpreadTopic, variable: mqtt.spread.variable.Variable):
        try:
            entity = self.get_entity(topic.entity_address.id)
            entity.on_command(topic, variable)

        except (errors.EntityNotPresent, errors.ClassifierError, errors.CommandError) as ex:
            logging.debug(ex)

            if REPLY_ON_ERROR and variable.session_id:
                to_topic = mqtt.spread.topic.Reply(self.id, variable.session_id)
                self.publish(to_topic, dict(error=dict(code=123, message='{}'.format(ex))), False)

    def on_state(self, topic: mqtt.spread.topic.State, variable: mqtt.spread.variable.Variable):
        recipe = self.get_recipe(topic.entity_address.manager_id, topic.entity_address.id)

        if recipe:
            recipe.dispatch_state(topic, variable)

    def on_exit(self):
        super(Equipment, self).on_exit()

        for entity in self.all_entities():
            try:
                entity.set_invalid()
            except BaseException as ex:
                logging.exception(ex)


class LightingEquipment(Equipment):
    subginery_type = subgineries.lighting.Lighting
    state_topics = ('On', 'BrightnessLevel', 'Temperature', 'EngineryHoldTime',
                    'CurrentLuminosity', 'CurrentPresence', 'LastEvent', 'Color')
    enginery_types = [
        engineries.DimmingLight, engineries.SwitchingLight, engineries.TunableWhiteLight,
        engineries.PresenceSensor, engineries.LightSensor,
        engineries.LightingArea, engineries.RgbLight, engineries.RgbwLight
    ]

    def __init__(self, *args, **kwargs):
        super(LightingEquipment, self).__init__(*args, **kwargs)
        self.button_presets = dict()

    def on_local(self, topic: mqtt.project.Preset, data):
        assert topic.entity_type == engineries.lighting_area.ButtonPreset.__name__

        if data:
            self.button_presets.setdefault(topic.entity_id, engineries.lighting_area.ButtonPreset(topic.entity_id))
            self.button_presets[topic.entity_id].update(json.loads(data))
        else:
            self.button_presets.pop(topic.entity_id)

    def exploit_topics(self, action):
        super(LightingEquipment, self).exploit_topics(action)
        action(mqtt.project.Preset.broadcast(self.id, engineries.lighting_area.ButtonPreset.__name__, '#'))


class ShadingEquipment(Equipment):
    subginery_type = subgineries.shading.Shading
    state_topics = ('PositionLevel', 'Motion')
    enginery_types = [engineries.shade.Shutter, engineries.shade.Curtain, engineries.shade.Blind]


class AlarmEquipment(Equipment):
    subginery_type = subgineries.alarming.Alarm
    state_topics = ('Guard', 'Triggered')
    enginery_types = [
        engineries.alarm.FireSensor,
        engineries.alarm.IntruderSensor,
        engineries.alarm.LeakageSensor
    ]


class HandlingEquipment(Equipment):
    subginery_type = subgineries.handling.Handling
    enginery_types = [engineries.handle.Scenario]


class ClimateEquipment(Equipment):
    subginery_type = subgineries.climating.Climate
    state_topics = ('Temperature', 'TargetTemperature', 'On', 'Preset',
                    'Mode', 'FanSpeedMode', 'LouverMode')
    enginery_types = [
        engineries.climate.TemperatureSensor,
        engineries.climate.Thermoregulator,
        engineries.climate.Fan,
        engineries.climate.HeatedFloor
    ]
